import { Hex, Abi, ContractFunctionArgs } from 'viem';
import { IntentSourceAbi } from '../abi';
/**
 * Extracts the functions from an ABI
 */
export type ExtractAbiFunctions<abi extends Abi> = Extract<abi[number], {
    type: 'function';
}>;
/**
 * Define the type for the Intent struct in the IntentSource
 */
export type IntentViewType = ContractFunctionArgs<typeof IntentSourceAbi, 'pure', 'getIntentHash'>[number];
/**
 * Define the type for the Route struct in IntentSource
 */
export type RouteViemType = IntentViewType['route'];
/**
 * Define the type for the Reward struct in IntentSource
 */
export type RewardViemType = IntentViewType['reward'];
/**
 * Encodes the route parameters
 * @param route the route to encode
 * @returns
 */
export declare function encodeRoute(route: RouteViemType): `0x${string}`;
/**
 * Encodes the reward parameters
 * @param reward the reward to encode
 * @returns
 */
export declare function encodeReward(reward: RewardViemType): `0x${string}`;
/**
 * Encodes the intent parameters
 * @param intent the intent to encode
 * @returns
 */
export declare function encodeIntent(intent: IntentViewType): `0x${string}`;
/**
 * Encodes a transferNative function call
 * @param to the address to send to
 * @param value the amount to send
 */
export declare function encodeTransferNative(to: Hex, value: bigint): Hex;
/**
 * Hashes the route of an intent
 * @param route the route to hash
 * @returns
 */
export declare function hashRoute(route: RouteViemType): Hex;
/**
 * Hashes the reward of an intent
 * @param reward the reward to hash
 * @returns
 */
export declare function hashReward(reward: RewardViemType): Hex;
/**
 * Hashes the intent and its sub structs
 * @param intent the intent to hash
 * @returns
 */
export declare function hashIntent(intent: IntentViewType): {
    routeHash: Hex;
    rewardHash: Hex;
    intentHash: Hex;
};
/**
 * Generates the intent vault address using CREATE2
 * @param intentSourceAddress the intent source address
 * @param intent the intent
 * @returns
 */
export declare function intentVaultAddress(intentSourceAddress: Hex, intent: IntentViewType): Hex;
